import _Array$from from 'babel-runtime/core-js/array/from';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _toConsumableArray from 'babel-runtime/helpers/toConsumableArray';
import _ from 'underscore';
import Map from 'es6-map';

export function isSubset(small, large) {
  if (!_.isArray(small) || !_.isArray(large)) {
    return false;
  }

  if (small.length > large.length) {
    return false;
  }

  for (var index = 0; index < small.length; index++) {
    if (small[index] !== large[index]) {
      return false;
    }
  }

  return true;
}

function constructCache(seedObj, predicate) {
  var cache = new Map();

  function recurse(obj) {
    var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var k = predicate(obj);

    if (k) {
      cache.set(path, k);
    }

    if (_.isObject(obj)) {
      _.each(obj, function (value, key) {
        recurse(value, [].concat(_toConsumableArray(path), [key]));
      });
    }
  }

  recurse(seedObj);

  return cache;
}

var ObjectCache = function () {
  function ObjectCache(seedObj, predicate) {
    _classCallCheck(this, ObjectCache);

    if (seedObj instanceof Map) {
      this.cache = seedObj;
    } else if (_.isObject(seedObj) && _.isFunction(predicate)) {
      this.cache = constructCache(seedObj, predicate);
    } else {
      this.cache = new Map();
    }
    this.getCacheForValue = this.getCacheForValue.bind(this);
    this.getCacheForPath = this.getCacheForPath.bind(this);
    this.getPaths = this.getPaths.bind(this);
    this.getValues = this.getValues.bind(this);
  }

  /**
   * Returns a subset of the cache graph filtered to only include the values equal to the query
   * @param query
   * @returns ObjectCache
   */


  _createClass(ObjectCache, [{
    key: 'getCacheForValue',
    value: function getCacheForValue(query) {
      var map = new Map();
      this.cache.forEach(function (value, path) {
        if (_.isEqual(query, value)) {
          map.set(path, value);
        }
      });

      return new ObjectCache(map);
    }

    /**
     * Returns a subset of the cache graph localized for the given path
     * @param currentPath
     * @returns ObjectCache
     */

  }, {
    key: 'getCacheForPath',
    value: function getCacheForPath(currentPath) {
      if (currentPath.length === 0) {
        return this;
      }

      var subset = _.partial(isSubset, currentPath);
      var relativePath = _.partial(_.drop, _, currentPath.length);
      var map = new Map();

      this.cache.forEach(function (value, path) {
        if (subset(path)) {
          map.set(relativePath(path), value);
        }
      });

      return new ObjectCache(map);
    }
  }, {
    key: 'getPaths',
    value: function getPaths() {
      return _Array$from(this.cache.keys());
    }
  }, {
    key: 'getValues',
    value: function getValues() {
      return _Array$from(this.cache.values());
    }
  }]);

  return ObjectCache;
}();

export default ObjectCache;
